@@ -95,6 +95,7 @@ from platon.types import (
     TxReceipt,
     Von,
     _Hash32,
+    UnFillTxParams,
 )
 
 
@@ -122,6 +123,13 @@ class BasePlaton(Module):
         mungers=[send_transaction_munger]
     )
 
+    fill_transaction_munger = send_transaction_munger
+
+    fill_transaction: Method[Callable[[UnFillTxParams], TxParams]] = Method(
+        RPC.platon_fillTransaction,
+        mungers=[fill_transaction_munger]
+    )
+
     _get_transaction: Method[Callable[[_Hash32], TxData]] = Method(
         RPC.platon_getTransactionByHash,
         mungers=[default_root_munger]
